import asyncio
import json
import logging
import socket
import ssl
import time
import urllib.parse
from typing import List, Dict

class Yumikogram:
    def __init__(self, bot_token: str, api_id: int, api_hash: str):
        self.bot_token = bot_token
        self.api_id = api_id
        self.api_hash = api_hash
        self.server = 'api.telegram.org'
        self.port = 443
        self.ssl_context = ssl.create_default_context()
        self.ssl_context.check_hostname = False
        self.ssl_context.verify_mode = ssl.CERT_NONE
        self.last_update_id = None

    async def send_message(self, chat_id: int, text: str, **kwargs) -> Dict:
        url = f'https://{self.server}:{self.port}/bot{self.bot_token}/sendMessage'
        data = {'chat_id': chat_id, 'text': text}
        data.update(kwargs)
        response = await self._send_request(url, data)
        return response.get('result', {})

    async def get_updates(self, **kwargs) -> List[Dict]:
        url = f'https://{self.server}:{self.port}/bot{self.bot_token}/getUpdates'
        data = {'timeout': 60}
        if self.last_update_id:
            data['offset'] = self.last_update_id + 1
        data.update(kwargs)
        response = await self._send_request(url, data)
        updates = response.get('result', [])
        for update in updates:
            self.last_update_id = update['update_id']
        return updates

    async def _send_request(self, url: str, data: Dict) -> Dict:
        encoded_data = urllib.parse.urlencode(data).encode('utf-8')
        request = f'POST {url} HTTP/1.1\r\nHost: {self.server}\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: {len(encoded_data)}\r\n\r\n{encoded_data.decode("utf-8")}'
        with socket.create_connection((self.server, self.port)) as sock:
            with self.ssl_context.wrap_socket(sock, server_hostname=self.server) as ssl_sock:
                ssl_sock.sendall(request.encode())
                response = b''
                while True:
                    chunk = ssl_sock.recv(4096)
                    if not chunk:
                        break
                    response += chunk
                response = response.decode('utf-8')
                response_headers, response_body = response.split('\r\n\r\n', 1)
                response_headers_dict = {}
                for header in response_headers.split('\r\n')[1:]:
                    name, value = header.split(': ', 1)
                    response_headers_dict[name] = value
                response_data = json.loads(response_body)
                if response_headers_dict.get('Content-Type', '').startswith('application/json'):
                    if not response_data.get('ok', False):
                        logging.error(response_data.get('description', ''))
                    return response_data
                else:
                    logging.error(f'Invalid response: {response_headers_dict.get("Content-Type", "")}')
        return {}




from setuptools import setup
import setuptools
import re

with open("requirements.txt", encoding="utf-8") as r:
    requires = [i.strip() for i in r]

with open("yumikogram/__init__.py", encoding="utf-8") as f:
    ver = re.findall(r"__version__ = \"(.+)\"", f.read())[0]

with open('README.md', 'r') as fh:
    long_description = fh.read()
    
setup(
    name='yumikogram',
    version=ver,
    description='This package for telegram bots making and api making',
    author='santhosh',
    author_email='yumikoapi@gmail.com',
    package_data={"yumikogram": ["py.typed"]},
    url = 'https://github.com/yumiko-api/yumikogram',
    long_description=long_description,
    packages=setuptools.find_packages(),
    keywords=['yumikogram', 'telegram', 'telegram-bots', 'yumikogram api', 'telegram bots library'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet",
        "Topic :: Communications",
        "Topic :: Communications :: Chat",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries :: Application Frameworks"
    ],
    python_requires='>=3.6',
    py_modules=['yumikogram'],
    install_requires=requires,
)



__version__ = "0.0.1"
__license__ = "GNU Lesser General Public License v3.0 (LGPL-3.0)"
__copyright__ = "Copyright (C) 2023-present yumiko-api <https://github.com/yumiko-api>"

def yumiko_api():
  print(
     f'Yumikogram Successfully installed Current version of Yumikogram {__version__}' + "\n"
     f'Under {__license__}'
  )

